warning off all
% solve the transition path for a transitory rise in uncertainty: benchmark case
% Zhang Zhewei
% 2022/3/8
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
var thet1t thet2t gt taxt wlt wkt rkt rhot epsilonstart qt Lambdat Yt ykt ylt rbt xkt xlt nkt nlt Pikt Pilt ct kt it bkt blt bonddt bondft ckt clt gamma1t gamma2t epsilonmint thetmint zt Ut Wt nvt;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
varexo eps_bonddt eps_bondft eps_gamma1t eps_gamma2t eps_zt eps_nvt;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parameters thetmin gamma1 gamma2 alphaone alphatwo alphathree zeta phi beta nv del psi kappa1 kappa2 epsilonmin tau bondd bondf z epsilonss qss rbf gss;
load ParaSet
set_param_value('thetmin',thetmin);
set_param_value('gamma1',gamma1);
set_param_value('gamma2',gamma2 );
set_param_value('alphaone',alphaone);
set_param_value('alphatwo',alphatwo);
set_param_value('alphathree',alphathree);
set_param_value('zeta',zeta);
set_param_value('phi',phi);
set_param_value('beta',beta);
set_param_value('nv',nv);
set_param_value('del',del);
set_param_value('psi',psi);
set_param_value('kappa1',kappa1);
set_param_value('kappa2',kappa2);
set_param_value('epsilonmin',epsilonmin);
set_param_value('bondd',bondd);
set_param_value('bondf',bondf);
set_param_value('tau',tau);
set_param_value('z',z);
set_param_value('epsilonss',epsilonss);
set_param_value('qss',qss);
set_param_value('rbf',rbf);
set_param_value('gss',gss);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load SteadyState0
external_function (name = Phi_1, nargs = 3);
external_function (name = Phi_2, nargs = 3);
external_function (name = Phi_3, nargs = 3);
external_function (name = Fun_g, nargs = 3);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
model;

% 1.Aggregate demand for final goods
ct = (1-qt)*xkt*Phi_2(thetmint,thet1t,gamma1t)+qt*xlt*Phi_2(thetmint,thet2t,gamma1t);

% 2.Consumption in K sector 
ckt = xkt*Phi_2(thetmint,thet1t,gamma1t);

% 3.Consumption in L sector
clt = xlt*Phi_2(thetmint,thet2t,gamma1t);

% 4.Aggregate demand for government bonds
bonddt = (1-qt)*xkt*Phi_3(thetmint,thet1t,gamma1t)+qt*xlt*Phi_3(thetmint,thet2t,gamma1t);

% 5.The budget constraint of government
(1+rbt(-1))*bonddt(-1)+(1+rbf)*bondft(-1)+gt = bonddt+bondft+taxt(-1);
%gt = gss;

% 6.The disposable income of household in sector K 
%xkt = (1+rbt(-1))*bonddt(-1)+(rkt+1-del)*kt(-1)-kt+Pikt/(1-qt)-nv+gt-taxt+wkt*nkt/(1-qt);
xkt = (1+rbt(-1))*bonddt(-1)+(rkt+1-del)*kt(-1)-kt+(1-tau)*(Pikt/(1-qt)-nvt+wkt*nkt/(1-qt));

% 7.
epsilonmint = 1-1/gamma2t;

% 8.The disposable income of household in sector L
xlt = (1+rbt(-1))*bonddt(-1)+(rkt+1-del)*kt(-1)-kt+(1-tau)*(Pilt/qt+wlt*nlt/qt)+gt/qt;

% 9.Aggregate demand for labor in sector K
nkt = alphatwo*ykt/wkt;

% 10.Supply of labor for sector K
phi = wkt*beta*(1+rbt)*Lambdat(+1)*Phi_1(thetmint,thet1t,gamma1t);

% 11.The monopoly profit of firms in sector K
Pikt = zeta*(zt*rhot)^(1/zeta)*(gamma2t/(gamma2t-1/zeta))*epsilonmint^(gamma2t)*epsilonstart^(1/zeta-gamma2t);

% 12.The monopoly profit of firms in sector L
Pilt = (1-alphathree)*ylt;

% 13.Aggregate demand for labor in sector L
nlt = alphathree*ylt/wlt;

% 14.Supply of labor in sector L
phi = wlt*beta*(1+rbt)*Lambdat(+1)*Phi_1(thetmint,thet2t,gamma1t);

% 15.Aggregate demand for physical assets
kt = alphaone*ykt/rkt;

% 16.The supply of physical assets
Lambdat = beta*(rkt(+1)+1-del)*Lambdat(+1);

% 17.The cutoff value of agents working in sector K
thet1t*beta*(1+rbt)*xkt*Lambdat(+1) = 1;

% 18.The cutoff value of agents working in sector L
thet2t*beta*(1+rbt)*xlt*Lambdat(+1) = 1;

% 19.The cutoff value for demand shock
epsilonstart = (nv/zeta)^zeta/zt/rhot;
%epsilonstart = epsilonss;

% 21.The discount factor
Lambdat=(1-qt)*phi/wkt+qt*phi/wlt;

% 22.The mass of agents working in sector 1
qt = Fun_g(epsilonstart,epsilonmint,gamma2t);
%qt = qss;

% 23.The definition of rhot
rhot = (alphaone/rkt)^alphaone*(alphatwo/wkt)^alphatwo;

% 24. Process of bond supply
log(bonddt)-log(bondd) = kappa2*(log(bonddt(-1))-log(bondd))+eps_bonddt;

% 25. Investment
it = kt-(1-del)*kt(-1);

% 26. Bond holding of k-sector
bkt = xkt*Phi_3(thetmint,thet1t,gamma1t);

% 27. Bond holding of l-sector
blt = xlt*Phi_3(thetmint,thet2t,gamma1t);

% 28. Process of tax rate
taxt = tau*(wkt*nkt+Pikt+wlt*nlt+Pilt-(1-qt)*nvt);

% 29. Process of income uncertainty
log(gamma1t)-log(gamma1) = kappa2*(log(gamma1t(-1))-log(gamma1))+eps_gamma1t;

% 30. 
thetmint = 1-1/gamma1t;

% 31. Process of demand shock
log(gamma2t)-log(gamma2) = kappa1*(log(gamma2t(-1))-log(gamma2))+eps_gamma2t;

% 32. Process of aggregate shock
log(zt)-log(z) = kappa1*(log(zt(-1))-log(z))+eps_zt;

% 33. The aggregate output in K sector
ykt = (zt*rhot)^(1/zeta)*(gamma2t/(gamma2t-1/zeta))*epsilonmint^(gamma2t)*epsilonstart^(1/zeta-gamma2t);

% 34. The aggregate output in L sector
ylt = psi*(alphathree*psi/wlt)^(alphathree/(1-alphathree));

% 35. The aggregate output in the economy
Yt = ykt+ylt; 

% 36. Overall Utility level
Ut = log(thetmint)+1/gamma1t-(1-qt)/gamma1t*(thetmint/thet1t)^(gamma1t)-qt/gamma1t*(thetmint/thet2t)^(gamma1t)+(1-qt)*log(xkt)+qt*log(xlt)-nkt-nlt;

% 37. The definition of welfare
Wt = Ut + beta*Wt(+1);

% 38. The supply of bonds issuing to foreign investors
log(bondft)-log(bondf) = kappa1*(log(bondft(-1))-log(bondf))+eps_bondft;

% 39. The fixed cost
log(nvt)-log(nv) = kappa1*(log(nvt(-1))-log(nv))+eps_nvt;

end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
initval;
thet1t = thet10; thet2t = thet20; gt = g0; taxt = tax0; wlt = wl0; wkt = wk0; rkt = rk0; rhot = rho0; epsilonstart = epsilonstar0;
qt = q0; Lambdat = Lambda0; Yt = Y0; ykt = yk0; ylt = yl0; rbt = rb0; xkt = xk0; xlt = xl0; nkt = nk0; nlt = nl0; 
Pikt = Pik0; Pilt = Pil0; ct = c0; kt = k0; it = i0; bkt = bk0; blt = bl0; bonddt = bondd; bondft = bondf; ckt = ck0; clt = cl0; gamma1t = gamma1; gamma2t = gamma2; epsilonmint = epsilonmin; 
thetmint = thetmin; zt = z; nvt = nv;
end;
steady;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
shocks;
var eps_zt;
periods 1;
values -0.1;
%var eps_nvt;
%periods 1;
%values 0.01;
var eps_bonddt;
periods 1;
values 0.1;
end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%simul(periods=2000);
simul(periods=3000,maxit=50000,stack_solve_algo=0);
%perfect_foresight_setup(periods=1000);
%perfect_foresight_solver(stack_solve_algo=0);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
